/******************************************************
STM8S103F2P6 16MHz, IAR1.4+STM8S_StdPeriph_Lib_V2.1.0
******************************************************/
#include "stm8s.h"
#include "iostm003.c"

#define _bitset(bits) ((\
  (bits%010)|\
  (bits/010%010)<<1|\
  (bits/0100%010)<<2|\
  (bits/01000%010)<<3|\
  (bits/010000%010)<<4|\
  (bits/0100000%010)<<5|\
  (bits/01000000%010)<<6|\
  (bits/010000000%010)<<7))
#define binary(bits) _bitset(0##bits) //  u8 a = BINARY(01100011);

#define R_upr 15000 //Ohms
#define R_lwr 5100   //Ohms
#define Ub_max_adc ((500UL*(R_upr+R_lwr))/R_lwr) //=U_max*100,19.7V

#define LEDA       PD_ODR_ODR5
#define button0 	 PD_IDR_IDR6
#define from_DCDC	 PA_ODR_ODR3

enum btn_states {b_prs, b_rls, b_hold} btn0=b_rls;
enum dmodes {normal,started,lobat,sdn,sleep} d_mode=started;

volatile u8 tim0,tim1,tim2,tim3,tmr1;
u16 adc4val;
u8 bat_ok;

void gpio_cfg()
{

   PA_ODR=PB_ODR=PC_ODR=PD_ODR=0;
   PA_DDR=PB_DDR=PC_DDR=PD_DDR=binary(11111111); //conf as output
   PA_CR1=PB_CR1=PC_CR1=PD_CR1=binary(11111111); //push-pull
   PA_CR2=PB_CR2=PC_CR2=PD_CR2=0;                //speed 2MHz
      
   PB_DDR_DDR5=PB_DDR_DDR4=0; PB_CR1_C15=PB_CR1_C14=1;
	 PD_DDR_DDR2=PD_DDR_DDR3=0; PD_CR1_C12=PD_CR1_C13=0;
   //PC_ODR_ODR5 = 1;
   PD_DDR_DDR6 = 0; PD_CR1_C16 = 1;
   
}
void clk_cfg()
{
  CLK_DeInit();
	CLK->CKDIVR = 0;
	CLK->PCKENR1 = binary(10000000); //only tim1 clk enabled
}
void iwdg_cfg()
{
  IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
  IWDG_SetPrescaler(IWDG_Prescaler_128);
  IWDG_SetReload((uint8_t)(128000/(128*250)));
  IWDG_ReloadCounter();
  /* Enable IWDG (the LSI oscillator will be enabled by hardware) */
  IWDG_Enable();
  IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);
  IWDG_ReloadCounter();
  
}
void t1_cfg()
{
#define arlv 0xff
  {
  /* Set the Autoreload value */	
  TIM1->ARRH = arlv >> 8;
  TIM1->ARRL = arlv;
  
	/* Select the Counter Mode */
	TIM1->CR1 = TIM1_COUNTERMODE_CENTERALIGNED1|TIM1_CR1_ARPE|TIM1_CR1_URS;
	TIM1->CR2 = TIM1_CR2_CCPC; 	//preload ccmpr1
	TIM1->RCR = 1;						//repetition counter for center aligned pwm mode
  TIM1->IER = TIM1_IER_UIE;		//update interrupt
	//TIM1->IER|= TIM1_IER_BIE; //break interrupt 
  }

  TIM1->CCER1=TIM1->CCER2=TIM1->CCMR1=TIM1->CCMR2=0;
	
  TIM1->CCER1|= TIM1_CCER1_CC1E|TIM1_CCER1_CC1NE|TIM1_CCER1_CC1NP;
  TIM1->CCER1|= TIM1_CCER1_CC2E|TIM1_CCER1_CC2NE|TIM1_CCER1_CC2NP;
  TIM1->CCMR1|= TIM1_OCMODE_PWM1;
  TIM1->CCMR2|= binary(01000000);  //force OC2ref inactive level;
  TIM1->EGR  |= TIM1_EGR_COMG;     //update CCER reg with COM event 
	

	
	TIM1->OISR = binary(0001010);		 
	TIM1->DTR  = 0x30; //3us dt delay (62,5ns*48)
	TIM1->CR1 |= TIM1_CR1_CEN;
 	TIM1->BKR  = TIM1_BKR_MOE|TIM1_BKR_BKE|TIM1_BKR_OSSR|TIM1_BKR_OSSI|TIM1_LOCKLEVEL_2;

}
void adc_cfg()
{
  PD_ODR_ODR3=0;
  
  ADC1_DeInit();
  ADC1_Init(ADC1_CONVERSIONMODE_SINGLE,ADC1_CHANNEL_4, 
            ADC1_PRESSEL_FCPU_D18, ADC1_EXTTRIG_TIM, DISABLE, ADC1_ALIGN_RIGHT,
            ADC1_SCHMITTTRIG_CHANNEL4, DISABLE);
  ADC1_StartConversion();
}
void afr_bits_set ()
{
	u8 a = binary(10010001);
	
	if(FLASH_ReadOptionByte(0x4803)!=((u16)a<<8|(~a)));
	{
    /* Unlock data memory */
   FLASH->DUKR = FLASH_RASS_KEY2;
   FLASH->DUKR = FLASH_RASS_KEY1;
	FLASH_ProgramOptionByte(0x4803,a);
	
	/* Lock memory */
	FLASH->IAPSR &= (uint8_t)FLASH_MEMTYPE_DATA;
	}
}

void delay(volatile uint16_t nCount)
{
    while (nCount)
    {
        nCount--;
        IWDG_ReloadCounter();
    }
}
void scan_keys ()
{
static volatile u8 count0,count1;

if (btn0 == b_rls){count1 = 0;}
if (!button0) {count0++;} else {count0 = 0; btn0 = b_rls;}
if ((count0 > 250)&&(btn0 != b_hold)) {btn0 = b_prs; count0 = 0; count1++;}
if ((count1 > 10)&&(btn0 == b_prs)) {btn0 = b_hold; count1 = 0;}
}
void lblk(u8 tlight, u8 tdark, u8 *j)
{
 static u8 stage;
 
 if(*j>9){*j=11;} 
 if(*j)
 {
 if((!tim0)&&(!stage)){tim0=tlight; LEDA=1; stage=1;}
 else if ((!tim0)&&(stage)){tim0=tdark; stage=LEDA=0; --*j;}
 }
 else {stage=LEDA=0;}
}
void ADC_read()
{
  static u8 counter4;
  static u16 tmp4=(u16)(127UL*1024/1970);
  
  if(ADC1_GetFlagStatus(ADC1_FLAG_EOC))
  {
   {
     if(counter4){counter4--;tmp4+=ADC1_GetConversionValue();}
     else {counter4=16;adc4val=tmp4/(counter4+1);tmp4=ADC1_GetConversionValue();}
   }
   ADC1_ClearFlag(ADC1_FLAG_EOC);
   ADC1_StartConversion(); 
  }
}

void dev_mgmt();void bat_detect();


INTERRUPT_HANDLER(TIM1_UPD_OVF_TRG_BRK_IRQHandler, 11)
{
#define s_step 314 
static u8 i=0;
static u8 rck=0, dir=0;	

static const u8 s_table [s_step/2]=
{
0,2,5,7,10,12,15,17,20,22,25,27,30,32,35,37,40,42,45,47,50,
52,54,57,59,62,64,67,69,72,74,76,79,81,84,86,88,91,93,95,98,100,
102,104,107,109,111,114,116,118,120,122,125,127,129,131,133,135,138,140,142,144,146,
148,150,152,154,156,158,160,162,164,166,167,169,171,173,175,177,178,180,182,184,185,
187,189,190,192,193,195,197,198,200,201,203,204,206,207,208,210,211,213,214,215,216,
218,219,220,221,223,224,225,226,227,228,229,230,231,232,233,234,235,236,236,237,238,
239,240,240,241,242,242,243,244,244,245,245,246,246,247,247,247,248,248,249,249,249,
249,250,250,250,250,250,250,250,250,250};                           


 if(!i)                                          
 {
 dir=1;i=1; 
   if(rck)
   {
    rck=0; 
    TIM1->CCMR1 = TIM1_OCMODE_PWM1; 
		TIM1->CCMR2 = binary(01000000); //force OC2ref inactive level
    if(tim0)tim0--;
    if(tim1)tim1--;
   }
   else
   {
    rck=1;
		TIM1->CCMR1 = binary(01000000); //force OC1ref inactive level 
		TIM1->CCMR2 = TIM1_OCMODE_PWM1;
    if(tim2)tim2--;
    if(tim3)tim3--;
   } 
   if(tmr1){tmr1--;} 
  TIM1->EGR|=TIM1_EGR_COMG;
 }
 else if((dir)&&(i<s_step/2-1))
 {
   TIM1->CCR1H=TIM1->CCR2H=0;
   TIM1->CCR1L=TIM1->CCR2L=s_table[i];
   i++;
 }
 else 
 {
   dir=0; i--; 
   TIM1->CCR1H=TIM1->CCR2H=0;
   TIM1->CCR1L=TIM1->CCR2L=s_table[i];
 }
  IWDG_ReloadCounter();
 
	TIM1->SR1 = 0;	//clear interrupts flags	
}



void main()
{

gpio_cfg();
clk_cfg();
adc_cfg();
t1_cfg();
iwdg_cfg();
afr_bits_set();

BEEP_Init(BEEP_FREQUENCY_2KHZ);
BEEP_Cmd(DISABLE);


asm("rim");


  while(1)
  {
    scan_keys ();
    dev_mgmt();
		


    
  }



}

void assert_failed(uint8_t* file, uint32_t line)
{

}

void bat_detect()
{
 u16 tmp;
 
   tmp=((unsigned long)adc4val*Ub_max_adc)/(0x3ff*10);
     
   if(tmp>108){bat_ok=1;tmr1=250;}   
   else if (tmp<90){bat_ok=0;TIM1->EGR = TIM1_EGR_BG;}
   if(!tmr1)  {bat_ok=0;}
   
}

void dev_mgmt()    
{
  	volatile static u16 tts,ttl=0;
	static u8 lbl;
  	static u8 bt;
  
 if(btn0==b_rls){bt=0;}

 switch (d_mode)                
 {
  
  case normal:
  {
  if ((btn0==b_hold)&&(!bt)) 
	{
		BEEP_Cmd(ENABLE);
		delay(20000);
		BEEP_Cmd(DISABLE);
		delay(30000);
		BEEP_Cmd(ENABLE);
		delay(20000);
		BEEP_Cmd(DISABLE);
		delay(30000);
		BEEP_Cmd(ENABLE);
		delay(20000);
		BEEP_Cmd(DISABLE);			
		d_mode=sdn; bt=1;
	}
  	if(!bat_ok){d_mode=lobat;} 
  	ADC_read();
  	bat_detect();
			
		if(!(TIM1->BKR&TIM1_BKR_MOE))
 {
	  
	  d_mode = sleep;
		BEEP_Cmd(ENABLE);
		delay(30000);
		BEEP_Cmd(DISABLE);
		delay(60000);
		BEEP_Cmd(ENABLE);
		delay(30000);
		BEEP_Cmd(DISABLE);
		delay(60000);
		BEEP_Cmd(ENABLE);
		delay(30000);
		BEEP_Cmd(DISABLE);
		delay(60000);
		BEEP_Cmd(ENABLE);
		delay(30000);
		BEEP_Cmd(DISABLE);
 }

  }
  break; 
  
  case started:
  {
		u8 cnt=0;

		lbl=5;
		lblk(50, 50, &lbl);		
m:	tim2=100;
		if(cnt<3){cnt++;} 
		else
		{
		cnt=tim2=0; 
		BEEP_Cmd(ENABLE);
    delay(65000);
    BEEP_Cmd(DISABLE);
		delay(30000);
		BEEP_Cmd(ENABLE);
    delay(65000);
    BEEP_Cmd(DISABLE);
		goto n;
		}
		if(!bat_ok){while(tim2){ADC_read();bat_detect();}}
		if(!bat_ok){goto m;}		
    else
		{
    BEEP_Cmd(ENABLE);
    delay(65000);
    BEEP_Cmd(DISABLE);
    
    TIM1->BKR |= TIM1_BKR_MOE;
   	d_mode=normal;
		LEDA = 1;
		goto o;
		}
n:	
		d_mode=lobat;
o: 	break;	
  }
  break;
  
  case lobat:
  {
  ADC_read();bat_detect();
	lbl=5;
	lblk(10, 50, &lbl);
        
  if(bat_ok){d_mode=started;}

  if(ttl){ttl--;}  
  if(!tts){tts=5;}
  else if((tts>2)&&(!ttl)){ttl=60000; tts--;}
  else if(tts<=2){tts=0;d_mode=sdn;}
  
  }
  break;
  
  case sdn:
  {
  TIM1->EGR = TIM1_EGR_BG;
	lbl=5;
	lblk(10, 100, &lbl);
  

  if (((btn0==b_prs)||(btn0==b_hold))&&(!bt)){d_mode=started;bt=1;}
  
  if(ttl){ttl--;}  
  if(!tts){tts=5;}
  else if((tts>2)&&(!ttl)){ttl=60000; tts--;}
  else if(tts<=2){tts=0;d_mode=sleep;}
  
  }
  break;
  
  case sleep:
  {
	lbl=5;
	lblk(10, 250, &lbl);

  if (((btn0==b_prs)||(btn0==b_hold))&&(!bt)){d_mode=started;bt=1;}  
  }
  break;

    
  default: break;
 }
}